@@ -12,7 +12,8 @@ class DataFormatChecker:
 
     def check_file_format(self, file_path: Path) -> None:
         """Verify that the file at file_path is one of the accepted_file_types."""
-        if file_path.suffix not in self.accepted_file_types:
+        file_type = "".join(file_path.suffixes)
+        if file_type not in self.accepted_file_types:
             raise ValueError(
                 "Invalid file format: '{}'. Input must be one of the "
                 "following: {}".format(file_path, self.accepted_file_types)
