@@ -1,54 +1,67 @@
 """Library for converting Protobufs to Python data types such as dataframes."""
 
-import re
-from typing import Dict, Optional
+from typing import Dict
 
+import numpy as np
 import pandas as pd
+from google.protobuf import empty_pb2
 from google.protobuf.json_format import MessageToDict
 
-from rime_sdk.protos.test_run_results.test_run_results_pb2 import TestRun
-
-MODEL_PERF_REGEX = r"^metrics\.model_perf.*(ref_metric|eval_metric)$"
+from rime_sdk.protos.result_synthesizer.result_message_pb2 import TestCaseMetricCategory
+from rime_sdk.protos.test_run_results.test_run_results_pb2 import (
+    TestBatchResult,
+    TestCase,
+    TestRunDetail,
+)
 
 # Map of flattened field paths to their types in the Dataframe.
 default_test_run_column_info = {
     # Metadata.
-    "metadata.id": "str",
-    "metadata.name": "str",
-    "metadata.project_id": "str",
-    "metadata.model_task": "str",  # OPTIONAL
+    "test_run_id": "str",
+    "name": "str",
+    "project_id": "str",
+    "model_task": "str",  # OPTIONAL
     # The canonical JSON encoding converts enums to their string representations
     # so we do not need to do manual conversions.
-    "metadata.data_type": "str",
-    "metadata.testing_type": "str",
-    "metadata.upload_time": "str",
+    "data_type": "str",
+    "testing_type": "str",
+    "upload_time": "str",
     # Model source info.
     "model_source_info.name": "str",  # OPTIONAL
     # Data source info.
     "data_source_info.ref.name": "str",
     "data_source_info.eval.name": "str",
     # Metrics.
-    "metrics.duration_millis": "float",
+    "metrics.duration_millis": "Int64",
     "metrics.num_inputs": "Int64",
     "metrics.num_failing_inputs": "Int64",
     "metrics.summary_counts.total": "Int64",
     "metrics.summary_counts.pass": "Int64",
     "metrics.summary_counts.warning": "Int64",
     "metrics.summary_counts.fail": "Int64",
-    "metrics.severity_counts.none": "Int64",
-    "metrics.severity_counts.low": "Int64",
-    "metrics.severity_counts.medium": "Int64",
-    "metrics.severity_counts.high": "Int64",
+    "metrics.summary_counts.skip": "Int64",
+    "metrics.severity_counts.num_none_severity": "Int64",
+    "metrics.severity_counts.num_low_severity": "Int64",
+    "metrics.severity_counts.num_medium_severity": "Int64",
+    "metrics.severity_counts.num_high_severity": "Int64",
 }
 
+# TestBatchResult columns that need to be converted from string to int64
+INT_TEST_BATCH_ATTRIBUTES = [
+    "duration_in_millis",
+    "summary_counts.total",
+    "summary_counts.warning",
+    "summary_counts.pass",
+    "summary_counts.fail",
+    "summary_counts.skip",
+]
+
 # Separator to use when flattening JSON into a dataframe.
 # columns_to_keep definition relies on this separator.
-df_flatten_separator = "."
+DF_FLATTEN_SEPARATOR = "."
 
 
-def parse_test_run_metadata(
-    test_run: TestRun, version: Optional[str] = None,  # pylint: disable=W0613
-) -> pd.DataFrame:
+def parse_test_run_metadata(test_run: TestRunDetail) -> pd.DataFrame:
     """Parse test run metadata Protobuf message into a Pandas dataframe.
 
     The columns are not guaranteed to be returned in sorted order.
@@ -70,14 +83,13 @@ def parse_test_run_metadata(
 
     # Flatten out nested fields in the Protobuf message.
     # The DF column name will be the field path joined by the `df_flatten_separator.`
-    normalized_df = pd.json_normalize(test_run_dict, sep=df_flatten_separator)
+    normalized_df = pd.json_normalize(test_run_dict, sep=DF_FLATTEN_SEPARATOR)
 
     default_test_run_columns = list(default_test_run_column_info.keys())
 
     # Include the model perf columns with the set of DF columns.
     # These are metrics like "Accuracy" over the reference and eval datasets.
-    model_perf_columns = [c for c in normalized_df if re.match(MODEL_PERF_REGEX, c)]
-    all_test_run_columns = default_test_run_columns + model_perf_columns
+    all_test_run_columns = default_test_run_columns + list(normalized_df.columns)
 
     missing_columns = set(all_test_run_columns).difference(set(normalized_df.columns))
     intersect_df = normalized_df[
@@ -110,3 +122,43 @@ def parse_test_run_metadata(
     # This is fine because the dataframe access pattern will likely involve column
     # names more than integer indices.
     return full_df
+
+
+def parse_test_batch_result(raw_result: TestBatchResult) -> pd.Series:
+    """Parse test batch result into a series."""
+    result_dict = MessageToDict(
+        raw_result,
+        including_default_value_fields=True,
+        preserving_proto_field_name=True,
+    )
+    df = pd.json_normalize(result_dict, sep=DF_FLATTEN_SEPARATOR)
+
+    for key in INT_TEST_BATCH_ATTRIBUTES:
+        # Some nested fields such as `metrics.severity_counts.low` will be `None`
+        # because MessageToDict does not populate nested primitive fields with
+        # default values.
+        # Since some columns may be `None`, we must convert to `float` first.
+        # https://stackoverflow.com/questions/60024262/error-converting-object-string-to-int32-typeerror-object-cannot-be-converted  # pylint: disable=line-too-long
+        df[key] = df[key].astype("float").astype("Int64")
+
+    return df.squeeze(axis=0)
+
+
+def parse_test_case_result(raw_result: TestCase, unpack_metrics: bool = False) -> dict:
+    """Parse proto test case result to pythonic form."""
+    raw_dict = MessageToDict(
+        raw_result,
+        including_default_value_fields=True,
+        preserving_proto_field_name=True,
+    )
+    del raw_dict["metrics"]
+    if unpack_metrics:
+        for metric in raw_result.metrics:
+            category_string = TestCaseMetricCategory.Name(metric.category)
+            prefix = "TEST_CASE_METRIC_CATEGORY_"
+            category_string = category_string[len(prefix) :]
+            metric_value = getattr(metric, metric.WhichOneof("data"))
+            if isinstance(metric_value, empty_pb2.Empty):
+                metric_value = np.nan
+            raw_dict[f"{category_string}:{metric.metric}"] = metric_value
+    return raw_dict
